# Copyright (c) 2015 Cloudbase Solutions SRL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
import mock

from manila.share.drivers.windows import windows_utils
from manila import test


@ddt.ddt
class WindowsUtilsTestCase(test.TestCase):
    def setUp(self):
        self._remote_exec = mock.Mock()
        self._windows_utils = windows_utils.WindowsUtils(self._remote_exec)
        super(WindowsUtilsTestCase, self).setUp()

    def test_initialize_disk(self):
        self._windows_utils.initialize_disk(mock.sentinel.server,
                                            mock.sentinel.disk_number)

        cmd = ["Initialize-Disk", "-Number", mock.sentinel.disk_number]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_create_partition(self):
        self._windows_utils.create_partition(mock.sentinel.server,
                                             mock.sentinel.disk_number)

        cmd = ["New-Partition", "-DiskNumber",
               mock.sentinel.disk_number, "-UseMaximumSize"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_format_partition(self):
        self._windows_utils.format_partition(mock.sentinel.server,
                                             mock.sentinel.disk_number,
                                             mock.sentinel.partition_number)
        cmd = ("Get-Partition -DiskNumber %(disk_number)s "
               "-PartitionNumber %(partition_number)s | "
               "Format-Volume -FileSystem NTFS -Force -Confirm:$false" % {
                   'disk_number': mock.sentinel.disk_number,
                   'partition_number': mock.sentinel.partition_number,
               })
        self._remote_exec.assert_called_once_with(mock.sentinel.server,
                                                  cmd)

    def test_add_access_path(self):
        self._windows_utils.add_access_path(mock.sentinel.server,
                                            mock.sentinel.mount_path,
                                            mock.sentinel.disk_number,
                                            mock.sentinel.partition_number)

        cmd = ["Add-PartitionAccessPath", "-DiskNumber",
               mock.sentinel.disk_number,
               "-PartitionNumber", mock.sentinel.partition_number,
               "-AccessPath", self._windows_utils.quote_string(
                   mock.sentinel.mount_path)
               ]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_resize_partition(self):
        self._windows_utils.resize_partition(mock.sentinel.server,
                                             mock.sentinel.size_bytes,
                                             mock.sentinel.disk_number,
                                             mock.sentinel.partition_number)

        cmd = ['Resize-Partition', '-DiskNumber', mock.sentinel.disk_number,
               '-PartitionNumber', mock.sentinel.partition_number,
               '-Size', mock.sentinel.size_bytes]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @ddt.data("1", "")
    def test_get_disk_number_by_serial_number(self, disk_number):
        mock_serial_number = "serial_number"
        self._remote_exec.return_value = (disk_number, mock.sentinel.std_err)
        expected_disk_number = int(disk_number) if disk_number else None

        result = self._windows_utils.get_disk_number_by_serial_number(
            mock.sentinel.server,
            mock_serial_number)

        pattern = "%s*" % mock_serial_number
        cmd = ("Get-Disk | "
               "Where-Object {$_.SerialNumber -like '%s'} | "
               "Select-Object -ExpandProperty Number" % pattern)

        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(expected_disk_number, result)

    @ddt.data("1", "")
    def test_get_disk_number_by_mount_path(self, disk_number):
        fake_mount_path = "fake_mount_path"
        self._remote_exec.return_value = (disk_number, mock.sentinel.std_err)
        expected_disk_number = int(disk_number) if disk_number else None

        result = self._windows_utils.get_disk_number_by_mount_path(
            mock.sentinel.server,
            fake_mount_path)

        cmd = ('Get-Partition | '
               'Where-Object {$_.AccessPaths -contains "%s"} | '
               'Select-Object -ExpandProperty DiskNumber' %
               (fake_mount_path + "\\"))
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(expected_disk_number, result)

    def test_get_volume_path_by_mount_path(self):
        fake_mount_path = "fake_mount_path"
        fake_volume_path = "fake_volume_path"
        self._remote_exec.return_value = fake_volume_path + '\r\n', None

        result = self._windows_utils.get_volume_path_by_mount_path(
            mock.sentinel.server,
            fake_mount_path)

        cmd = ('Get-Partition | '
               'Where-Object {$_.AccessPaths -contains "%s"} | '
               'Get-Volume | '
               'Select-Object -ExpandProperty Path' %
               (fake_mount_path + "\\"))
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(fake_volume_path, result)

    def test_get_disk_space_by_path(self):
        fake_disk_size = 1024
        fake_free_bytes = 1000
        fake_fsutil_output = ("Total # of bytes  : %(total_bytes)s"
                              "Total # of avail free bytes  : %(free_bytes)s"
                              % dict(total_bytes=fake_disk_size,
                                     free_bytes=fake_free_bytes))
        self._remote_exec.return_value = fake_fsutil_output, None

        result = self._windows_utils.get_disk_space_by_path(
            mock.sentinel.server,
            mock.sentinel.mount_path)

        cmd = ["fsutil", "volume", "diskfree",
               self._windows_utils.quote_string(mock.sentinel.mount_path)]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual((fake_disk_size, fake_free_bytes), result)

    def test_get_partition_maximum_size(self):
        fake_max_size = 1024
        self._remote_exec.return_value = ("%s" % fake_max_size,
                                          mock.sentinel.std_err)

        result = self._windows_utils.get_partition_maximum_size(
            mock.sentinel.server,
            mock.sentinel.disk_number,
            mock.sentinel.partition_number)

        cmd = ('Get-PartitionSupportedSize -DiskNumber %(disk_number)s '
               '-PartitionNumber %(partition_number)s | '
               'Select-Object -ExpandProperty SizeMax' %
               dict(disk_number=mock.sentinel.disk_number,
                    partition_number=mock.sentinel.partition_number))
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(fake_max_size, result)

    def test_set_disk_online_status(self):
        self._windows_utils.set_disk_online_status(mock.sentinel.server,
                                                   mock.sentinel.disk_number,
                                                   online=True)

        cmd = ["Set-Disk", "-Number", mock.sentinel.disk_number,
               "-IsOffline", 0]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_set_disk_readonly_status(self):
        self._windows_utils.set_disk_readonly_status(mock.sentinel.server,
                                                     mock.sentinel.disk_number,
                                                     readonly=False)

        cmd = ["Set-Disk", "-Number", mock.sentinel.disk_number,
               "-IsReadOnly", 0]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_update_disk(self):
        self._windows_utils.update_disk(mock.sentinel.server,
                                        mock.sentinel.disk_number)

        cmd = ["Update-Disk", mock.sentinel.disk_number]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_join_domain(self):
        mock_server = {'ip': mock.sentinel.server_ip}

        self._windows_utils.join_domain(mock_server,
                                        mock.sentinel.domain,
                                        mock.sentinel.admin_username,
                                        mock.sentinel.admin_password)

        cmds = [
            ('$password = "%s" | '
             'ConvertTo-SecureString -asPlainText -Force' %
             mock.sentinel.admin_password),
            ('$credential = '
             'New-Object System.Management.Automation.PSCredential('
             '"%s", $password)' % mock.sentinel.admin_username),
            ('Add-Computer -DomainName "%s" -Credential $credential' %
             mock.sentinel.domain)]
        cmd = ";".join(cmds)
        self._remote_exec.assert_called_once_with(mock_server, cmd)

    def test_unjoin_domain(self):
        self._windows_utils.unjoin_domain(mock.sentinel.server,
                                          mock.sentinel.admin_username,
                                          mock.sentinel.admin_password)

        cmds = [
            ('$password = "%s" | '
             'ConvertTo-SecureString -asPlainText -Force' %
             mock.sentinel.admin_password),
            ('$credential = '
             'New-Object System.Management.Automation.PSCredential('
             '"%s", $password)' % mock.sentinel.admin_username),
            ('Remove-Computer -UnjoinDomaincredential $credential '
             '-Passthru -Verbose -Force')]
        cmd = ";".join(cmds)
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_get_current_domain(self):
        fake_domain = " domain"
        self._remote_exec.return_value = (fake_domain, mock.sentinel.std_err)

        result = self._windows_utils.get_current_domain(mock.sentinel.server)

        cmd = "(Get-WmiObject Win32_ComputerSystem).Domain"
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(fake_domain.strip(), result)

    def test_ensure_directory_exists(self):
        self._windows_utils.ensure_directory_exists(mock.sentinel.server,
                                                    mock.sentinel.path)

        cmd = ["New-Item", "-ItemType", "Directory", "-Force", "-Path",
               self._windows_utils.quote_string(mock.sentinel.path)]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @ddt.data(False, True)
    @mock.patch.object(windows_utils.WindowsUtils, 'path_exists')
    def test_remove(self, is_junction, mock_path_exists):
        recurse = True
        self._windows_utils.remove(mock.sentinel.server,
                                   mock.sentinel.path,
                                   is_junction=is_junction,
                                   recurse=recurse)

        if is_junction:
            cmd = ('[System.IO.Directory]::Delete('
                   '%(path)s, %(recurse)d)'
                   % dict(path=self._windows_utils.quote_string(
                          mock.sentinel.path),
                          recurse=recurse))
        else:
            cmd = ["Remove-Item", "-Confirm:$false", "-Path",
                   self._windows_utils.quote_string(mock.sentinel.path),
                   "-Force", '-Recurse']

        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @mock.patch.object(windows_utils.WindowsUtils, 'path_exists')
    def test_remove_unexisting_path(self, mock_path_exists):
        mock_path_exists.return_value = False
        self._windows_utils.remove(mock.sentinel.server,
                                   mock.sentinel.path)
        self.assertFalse(self._remote_exec.called)

    @ddt.data("True", "False")
    def test_path_exists(self, path_exists):
        self._remote_exec.return_value = (path_exists,
                                          mock.sentinel.std_err)

        result = self._windows_utils.path_exists(mock.sentinel.server,
                                                 mock.sentinel.path)

        cmd = ["Test-Path", mock.sentinel.path]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(path_exists == "True", result)

    def test_normalize_path(self):
        fake_path = "C:/"
        result = self._windows_utils.normalize_path(fake_path)

        self.assertEqual("C:\\", result)

    def test_get_interface_index_by_ip(self):
        _FAKE_INDEX = "2"
        self._remote_exec.return_value = (_FAKE_INDEX, mock.sentinel.std_err)

        result = self._windows_utils.get_interface_index_by_ip(
            mock.sentinel.server,
            mock.sentinel.ip)

        cmd = ('Get-NetIPAddress | '
               'Where-Object {$_.IPAddress -eq "%(ip)s"} | '
               'Select-Object -ExpandProperty InterfaceIndex' %
               dict(ip=mock.sentinel.ip))
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)
        self.assertEqual(int(_FAKE_INDEX), result)

    def test_set_dns_client_search_list(self):
        mock_search_list = ["A", "B", "C"]

        self._windows_utils.set_dns_client_search_list(mock.sentinel.server,
                                                       mock_search_list)

        cmd = ["Set-DnsClientGlobalSetting",
               "-SuffixSearchList", "@('A','B','C')"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_set_dns_client_server_addresses(self):
        mock_dns_servers = ["A", "B", "C"]

        self._windows_utils.set_dns_client_server_addresses(
            mock.sentinel.server,
            mock.sentinel.if_index,
            mock_dns_servers)

        cmd = ["Set-DnsClientServerAddress",
               "-InterfaceIndex", mock.sentinel.if_index,
               "-ServerAddresses", "('A','B','C')"]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    def test_set_win_reg_value(self):
        self._windows_utils.set_win_reg_value(mock.sentinel.server,
                                              mock.sentinel.path,
                                              mock.sentinel.key,
                                              mock.sentinel.value)

        cmd = ['Set-ItemProperty', '-Path',
               self._windows_utils.quote_string(mock.sentinel.path),
               '-Name', mock.sentinel.key, '-Value', mock.sentinel.value]
        self._remote_exec.assert_called_once_with(mock.sentinel.server, cmd)

    @ddt.data(None, mock.sentinel.key_name)
    def test_get_win_reg_value(self, key_name):
        self._remote_exec.return_value = (mock.sentinel.value,
                                          mock.sentinel.std_err)

        result = self._windows_utils.get_win_reg_value(mock.sentinel.server,
                                                       mock.sentinel.path,
                                                       name=key_name)

        cmd = "Get-ItemProperty -Path %s" % (
            self._windows_utils.quote_string(mock.sentinel.path))
        if key_name:
            cmd += " | Select-Object -ExpandProperty %s" % key_name
        self._remote_exec.assert_called_once_with(mock.sentinel.server,
                                                  cmd,
                                                  retry=False)
        self.assertEqual(mock.sentinel.value, result)

    def test_quote_string(self):
        result = self._windows_utils.quote_string(mock.sentinel.string)
        self.assertEqual('"%s"' % mock.sentinel.string, result)



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Mock unit tests for the NetApp driver protocols NFS class module.
"""

import copy
import uuid

import ddt
import mock

from manila import exception
from manila.share.drivers.netapp.dataontap.protocols import nfs_cmode
from manila import test
from manila.tests.share.drivers.netapp.dataontap.protocols \
    import fakes as fake


@ddt.ddt
class NetAppClusteredNFSHelperTestCase(test.TestCase):

    def setUp(self):
        super(NetAppClusteredNFSHelperTestCase, self).setUp()

        self.mock_context = mock.Mock()
        self.mock_client = mock.Mock()
        self.helper = nfs_cmode.NetAppCmodeNFSHelper()
        self.helper.set_client(self.mock_client)

    @ddt.data(('1.2.3.4', '1.2.3.4'), ('fc00::1', '[fc00::1]'))
    @ddt.unpack
    def test__escaped_address(self, raw, escaped):
        self.assertEqual(escaped, self.helper._escaped_address(raw))

    def test_create_share(self):

        mock_ensure_export_policy = self.mock_object(self.helper,
                                                     '_ensure_export_policy')
        self.mock_client.get_volume_junction_path.return_value = (
            fake.NFS_SHARE_PATH)

        result = self.helper.create_share(fake.NFS_SHARE, fake.SHARE_NAME)

        export_addresses = [fake.SHARE_ADDRESS_1, fake.SHARE_ADDRESS_2]
        export_paths = [result(address) for address in export_addresses]
        expected_paths = [
            fake.SHARE_ADDRESS_1 + ":" + fake.NFS_SHARE_PATH,
            fake.SHARE_ADDRESS_2 + ":" + fake.NFS_SHARE_PATH,
        ]
        self.assertEqual(expected_paths, export_paths)
        (self.mock_client.clear_nfs_export_policy_for_volume.
            assert_called_once_with(fake.SHARE_NAME))
        self.assertTrue(mock_ensure_export_policy.called)

    def test_delete_share(self):

        self.helper.delete_share(fake.NFS_SHARE, fake.SHARE_NAME)

        (self.mock_client.clear_nfs_export_policy_for_volume.
            assert_called_once_with(fake.SHARE_NAME))
        self.mock_client.soft_delete_nfs_export_policy.assert_called_once_with(
            fake.EXPORT_POLICY_NAME)

    def test_update_access(self):

        self.mock_object(self.helper, '_ensure_export_policy')
        self.mock_object(self.helper,
                         '_get_export_policy_name',
                         mock.Mock(return_value='fake_export_policy'))
        self.mock_object(self.helper,
                         '_get_temp_export_policy_name',
                         mock.Mock(side_effect=['fake_new_export_policy',
                                                'fake_old_export_policy']))

        self.helper.update_access(fake.CIFS_SHARE,
                                  fake.SHARE_NAME,
                                  [fake.IP_ACCESS])

        self.mock_client.create_nfs_export_policy.assert_called_once_with(
            'fake_new_export_policy')
        self.mock_client.add_nfs_export_rule.assert_called_once_with(
            'fake_new_export_policy', fake.CLIENT_ADDRESS_1, False)
        (self.mock_client.set_nfs_export_policy_for_volume.
            assert_called_once_with(fake.SHARE_NAME, 'fake_new_export_policy'))
        (self.mock_client.soft_delete_nfs_export_policy.
            assert_called_once_with('fake_old_export_policy'))
        self.mock_client.rename_nfs_export_policy.assert_has_calls([
            mock.call('fake_export_policy', 'fake_old_export_policy'),
            mock.call('fake_new_export_policy', 'fake_export_policy'),
        ])

    def test_validate_access_rule(self):

        result = self.helper._validate_access_rule(fake.IP_ACCESS)

        self.assertIsNone(result)

    def test_validate_access_rule_invalid_type(self):

        rule = copy.copy(fake.IP_ACCESS)
        rule['access_type'] = 'user'

        self.assertRaises(exception.InvalidShareAccess,
                          self.helper._validate_access_rule,
                          rule)

    def test_validate_access_rule_invalid_level(self):

        rule = copy.copy(fake.IP_ACCESS)
        rule['access_level'] = 'none'

        self.assertRaises(exception.InvalidShareAccessLevel,
                          self.helper._validate_access_rule,
                          rule)

    def test_get_target(self):

        target = self.helper.get_target(fake.NFS_SHARE)
        self.assertEqual(fake.SHARE_ADDRESS_1, target)

    def test_get_share_name_for_share(self):

        self.mock_client.get_volume_at_junction_path.return_value = (
            fake.VOLUME)

        share_name = self.helper.get_share_name_for_share(fake.NFS_SHARE)

        self.assertEqual(fake.SHARE_NAME, share_name)
        self.mock_client.get_volume_at_junction_path.assert_called_once_with(
            fake.NFS_SHARE_PATH)

    def test_get_share_name_for_share_not_found(self):

        self.mock_client.get_volume_at_junction_path.return_value = None

        share_name = self.helper.get_share_name_for_share(fake.NFS_SHARE)

        self.assertIsNone(share_name)
        self.mock_client.get_volume_at_junction_path.assert_called_once_with(
            fake.NFS_SHARE_PATH)

    def test_get_target_missing_location(self):

        target = self.helper.get_target({'export_location': ''})
        self.assertEqual('', target)

    def test_get_export_location(self):

        host_ip, export_path = self.helper._get_export_location(
            fake.NFS_SHARE)
        self.assertEqual(fake.SHARE_ADDRESS_1, host_ip)
        self.assertEqual('/' + fake.SHARE_NAME, export_path)

    @ddt.data('', 'invalid')
    def test_get_export_location_missing_location_invalid(self, export):

        fake_share = fake.NFS_SHARE.copy()
        fake_share['export_location'] = export

        host_ip, export_path = self.helper._get_export_location(fake_share)

        self.assertEqual('', host_ip)
        self.assertEqual('', export_path)

    def test_get_temp_export_policy_name(self):

        self.mock_object(uuid, 'uuid1', mock.Mock(return_value='fake-uuid'))

        result = self.helper._get_temp_export_policy_name()

        self.assertEqual('temp_fake_uuid', result)

    def test_get_export_policy_name(self):

        result = self.helper._get_export_policy_name(fake.NFS_SHARE)
        self.assertEqual(fake.EXPORT_POLICY_NAME, result)

    def test_ensure_export_policy_equal(self):

        self.mock_client.get_nfs_export_policy_for_volume.return_value = (
            fake.EXPORT_POLICY_NAME)

        self.helper._ensure_export_policy(fake.NFS_SHARE, fake.SHARE_NAME)

        self.assertFalse(self.mock_client.create_nfs_export_policy.called)
        self.assertFalse(self.mock_client.rename_nfs_export_policy.called)

    def test_ensure_export_policy_default(self):

        self.mock_client.get_nfs_export_policy_for_volume.return_value = (
            'default')

        self.helper._ensure_export_policy(fake.NFS_SHARE, fake.SHARE_NAME)

        self.mock_client.create_nfs_export_policy.assert_called_once_with(
            fake.EXPORT_POLICY_NAME)
        (self.mock_client.set_nfs_export_policy_for_volume.
            assert_called_once_with(fake.SHARE_NAME, fake.EXPORT_POLICY_NAME))
        self.assertFalse(self.mock_client.rename_nfs_export_policy.called)

    def test_ensure_export_policy_rename(self):

        self.mock_client.get_nfs_export_policy_for_volume.return_value = 'fake'

        self.helper._ensure_export_policy(fake.NFS_SHARE, fake.SHARE_NAME)

        self.assertFalse(self.mock_client.create_nfs_export_policy.called)
        self.mock_client.rename_nfs_export_policy.assert_called_once_with(
            'fake', fake.EXPORT_POLICY_NAME)



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2011 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg


from manila import test

CONF = cfg.CONF
CONF.register_opt(cfg.StrOpt('conf_unittest',
                             default='foo',
                             help='for testing purposes only'))


class ConfigTestCase(test.TestCase):

    def test_declare(self):
        self.assertNotIn('answer', CONF)
        CONF.import_opt('answer', 'manila.tests.declare_conf')
        self.assertIn('answer', CONF)
        self.assertEqual(42, CONF.answer)

        # Make sure we don't overwrite anything
        CONF.set_override('answer', 256)
        self.assertEqual(256, CONF.answer)
        CONF.import_opt('answer', 'manila.tests.declare_conf')
        self.assertEqual(256, CONF.answer)

    def test_runtime_and_unknown_flags(self):
        self.assertNotIn('runtime_answer', CONF)
        import manila.tests.runtime_conf  # noqa
        self.assertIn('runtime_answer', CONF)
        self.assertEqual(54, CONF.runtime_answer)

    def test_long_vs_short_flags(self):
        CONF.clear()
        CONF.register_cli_opt(cfg.StrOpt('duplicate_answer_long',
                                         default='val',
                                         help='desc'))
        CONF.register_cli_opt(cfg.IntOpt('duplicate_answer',
                                         default=50,
                                         help='desc'))

        argv = ['--duplicate_answer=60']
        CONF(argv, default_config_files=[])
        self.assertEqual(60, CONF.duplicate_answer)
        self.assertEqual('val', CONF.duplicate_answer_long)

    def test_flag_leak_left(self):
        self.assertEqual('foo', CONF.conf_unittest)
        self.flags(conf_unittest='bar')
        self.assertEqual('bar', CONF.conf_unittest)

    def test_flag_leak_right(self):
        self.assertEqual('foo', CONF.conf_unittest)
        self.flags(conf_unittest='bar')
        self.assertEqual('bar', CONF.conf_unittest)

    def test_flag_overrides(self):
        self.assertEqual('foo', CONF.conf_unittest)
        self.flags(conf_unittest='bar')
        self.assertEqual('bar', CONF.conf_unittest)
        CONF.reset()
        self.assertEqual('foo', CONF.conf_unittest)



# Copyright 2016 Nexenta Systems, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import ddt
from oslo_utils import units

from manila.share.drivers.nexenta import utils
from manila import test


@ddt.ddt
class TestNexentaUtils(test.TestCase):

    @ddt.data(
        # Test empty value
        (None, 0),
        ('', 0),
        ('0', 0),
        ('12', 12),
        # Test int values
        (10, 10),
        # Test bytes string
        ('1b', 1),
        ('1B', 1),
        ('1023b', 1023),
        ('0B', 0),
        # Test other units
        ('1M', units.Mi),
        ('1.0M', units.Mi),
    )
    @ddt.unpack
    def test_str2size(self, value, result):
        self.assertEqual(result, utils.str2size(value))

    def test_str2size_input_error(self):
        # Invalid format value
        self.assertRaises(ValueError, utils.str2size, 'A')



# Copyright (c) 2015 Clinton Knight.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Mock unit tests for the NetApp driver protocols CIFS class module.
"""

import copy

import ddt
import mock

from manila.common import constants
from manila import exception
from manila.share.drivers.netapp.dataontap.protocols import cifs_cmode
from manila import test
from manila.tests.share.drivers.netapp.dataontap.protocols \
    import fakes as fake


@ddt.ddt
class NetAppClusteredCIFSHelperTestCase(test.TestCase):

    def setUp(self):
        super(NetAppClusteredCIFSHelperTestCase, self).setUp()

        self.mock_context = mock.Mock()

        self.mock_client = mock.Mock()
        self.helper = cifs_cmode.NetAppCmodeCIFSHelper()
        self.helper.set_client(self.mock_client)

    def test_create_share(self):

        result = self.helper.create_share(fake.CIFS_SHARE, fake.SHARE_NAME)

        export_addresses = [fake.SHARE_ADDRESS_1, fake.SHARE_ADDRESS_2]
        export_paths = [result(address) for address in export_addresses]
        expected_paths = [
            r'\\%s\%s' % (fake.SHARE_ADDRESS_1, fake.SHARE_NAME),
            r'\\%s\%s' % (fake.SHARE_ADDRESS_2, fake.SHARE_NAME),
        ]
        self.assertEqual(expected_paths, export_paths)
        self.mock_client.create_cifs_share.assert_called_once_with(
            fake.SHARE_NAME)
        self.mock_client.remove_cifs_share_access.assert_called_once_with(
            fake.SHARE_NAME, 'Everyone')
        self.mock_client.set_volume_security_style.assert_called_once_with(
            fake.SHARE_NAME, security_style='ntfs')

    def test_delete_share(self):

        self.helper.delete_share(fake.CIFS_SHARE, fake.SHARE_NAME)

        self.mock_client.remove_cifs_share.assert_called_once_with(
            fake.SHARE_NAME)

    def test_update_access(self):

        mock_validate_access_rule = self.mock_object(self.helper,
                                                     '_validate_access_rule')
        mock_get_access_rules = self.mock_object(
            self.helper, '_get_access_rules',
            mock.Mock(return_value=fake.EXISTING_CIFS_RULES))
        mock_handle_added_rules = self.mock_object(self.helper,
                                                   '_handle_added_rules')
        mock_handle_ro_to_rw_rules = self.mock_object(self.helper,
                                                      '_handle_ro_to_rw_rules')
        mock_handle_rw_to_ro_rules = self.mock_object(self.helper,
                                                      '_handle_rw_to_ro_rules')
        mock_handle_deleted_rules = self.mock_object(self.helper,
                                                     '_handle_deleted_rules')

        self.helper.update_access(fake.CIFS_SHARE,
                                  fake.SHARE_NAME,
                                  [fake.USER_ACCESS])

        new_rules = {'fake_user': constants.ACCESS_LEVEL_RW}
        mock_validate_access_rule.assert_called_once_with(fake.USER_ACCESS)
        mock_get_access_rules.assert_called_once_with(fake.CIFS_SHARE,
                                                      fake.SHARE_NAME)
        mock_handle_added_rules.assert_called_once_with(
            fake.SHARE_NAME, fake.EXISTING_CIFS_RULES, new_rules)
        mock_handle_ro_to_rw_rules.assert_called_once_with(
            fake.SHARE_NAME, fake.EXISTING_CIFS_RULES, new_rules)
        mock_handle_rw_to_ro_rules.assert_called_once_with(
            fake.SHARE_NAME, fake.EXISTING_CIFS_RULES, new_rules)
        mock_handle_deleted_rules.assert_called_once_with(
            fake.SHARE_NAME, fake.EXISTING_CIFS_RULES, new_rules)

    def test_validate_access_rule(self):

        result = self.helper._validate_access_rule(fake.USER_ACCESS)

        self.assertIsNone(result)

    def test_validate_access_rule_invalid_type(self):

        rule = copy.copy(fake.USER_ACCESS)
        rule['access_type'] = 'ip'

        self.assertRaises(exception.InvalidShareAccess,
                          self.helper._validate_access_rule,
                          rule)

    def test_validate_access_rule_invalid_level(self):

        rule = copy.copy(fake.USER_ACCESS)
        rule['access_level'] = 'none'

        self.assertRaises(exception.InvalidShareAccessLevel,
                          self.helper._validate_access_rule,
                          rule)

    def test_handle_added_rules(self):

        self.helper._handle_added_rules(fake.SHARE_NAME,
                                        fake.EXISTING_CIFS_RULES,
                                        fake.NEW_CIFS_RULES)

        self.mock_client.add_cifs_share_access.assert_has_calls([
            mock.call(fake.SHARE_NAME, 'user5', False),
            mock.call(fake.SHARE_NAME, 'user6', True),
        ], any_order=True)

    def test_handle_ro_to_rw_rules(self):

        self.helper._handle_ro_to_rw_rules(fake.SHARE_NAME,
                                           fake.EXISTING_CIFS_RULES,
                                           fake.NEW_CIFS_RULES)

        self.mock_client.modify_cifs_share_access.assert_has_calls([
            mock.call(fake.SHARE_NAME, 'user2', False)
        ])

    def test_handle_rw_to_ro_rules(self):

        self.helper._handle_rw_to_ro_rules(fake.SHARE_NAME,
                                           fake.EXISTING_CIFS_RULES,
                                           fake.NEW_CIFS_RULES)

        self.mock_client.modify_cifs_share_access.assert_has_calls([
            mock.call(fake.SHARE_NAME, 'user3', True)
        ])

    def test_handle_deleted_rules(self):

        self.helper._handle_deleted_rules(fake.SHARE_NAME,
                                          fake.EXISTING_CIFS_RULES,
                                          fake.NEW_CIFS_RULES)

        self.mock_client.remove_cifs_share_access.assert_has_calls([
            mock.call(fake.SHARE_NAME, 'user4')
        ])

    def test_get_access_rules(self):

        self.mock_client.get_cifs_share_access = (
            mock.Mock(return_value='fake_rules'))

        result = self.helper._get_access_rules(fake.CIFS_SHARE,
                                               fake.SHARE_NAME)

        self.assertEqual('fake_rules', result)
        self.mock_client.get_cifs_share_access.assert_called_once_with(
            fake.SHARE_NAME)

    def test_get_target(self):

        target = self.helper.get_target(fake.CIFS_SHARE)
        self.assertEqual(fake.SHARE_ADDRESS_1, target)

    def test_get_target_missing_location(self):

        target = self.helper.get_target({'export_location': ''})
        self.assertEqual('', target)

    def test_get_share_name_for_share(self):

        share_name = self.helper.get_share_name_for_share(fake.CIFS_SHARE)

        self.assertEqual(fake.SHARE_NAME, share_name)

    @ddt.data(
        {
            'location': r'\\%s\%s' % (fake.SHARE_ADDRESS_1, fake.SHARE_NAME),
            'ip': fake.SHARE_ADDRESS_1,
            'share_name': fake.SHARE_NAME,
        }, {
            'location': r'//%s/%s' % (fake.SHARE_ADDRESS_1, fake.SHARE_NAME),
            'ip': fake.SHARE_ADDRESS_1,
            'share_name': fake.SHARE_NAME,
        },
        {'location': '', 'ip': '', 'share_name': ''},
        {'location': 'invalid', 'ip': '', 'share_name': ''},
    )
    @ddt.unpack
    def test_get_export_location(self, location, ip, share_name):

        share = fake.CIFS_SHARE.copy()
        share['export_location'] = location

        result_ip, result_share_name = self.helper._get_export_location(share)

        self.assertEqual(ip, result_ip)
        self.assertEqual(share_name, result_share_name)



# Copyright (c) 2015 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import excutils
from oslo_utils import strutils

from manila import exception
from manila.i18n import _
from manila.share.drivers.huawei import constants


class SmartPartition(object):
    def __init__(self, helper):
        self.helper = helper

    def add(self, opts, fsid):
        if not strutils.bool_from_string(opts['huawei_smartpartition']):
            return
        if not opts['partitionname']:
            raise exception.InvalidInput(
                reason=_('Partition name is None, please set '
                         'huawei_smartpartition:partitionname in key.'))

        partition_id = self.helper._get_partition_id_by_name(
            opts['partitionname'])
        if not partition_id:
            raise exception.InvalidInput(
                reason=_('Can not find partition id.'))

        self.helper._add_fs_to_partition(fsid, partition_id)


class SmartCache(object):
    def __init__(self, helper):
        self.helper = helper

    def add(self, opts, fsid):
        if not strutils.bool_from_string(opts['huawei_smartcache']):
            return
        if not opts['cachename']:
            raise exception.InvalidInput(
                reason=_('Illegal value specified for cache.'))

        cache_id = self.helper._get_cache_id_by_name(opts['cachename'])
        if not cache_id:
            raise exception.InvalidInput(
                reason=(_('Can not find cache id by cache name %(name)s.')
                        % {'name': opts['cachename']}))

        self.helper._add_fs_to_cache(fsid, cache_id)


class SmartQos(object):
    def __init__(self, helper):
        self.helper = helper

    def create_qos(self, qos, fs_id):
        policy_id = None
        try:
            # Check QoS priority.
            if self._check_qos_high_priority(qos):
                self.helper.change_fs_priority_high(fs_id)
            # Create QoS policy and activate it.
            (qos_id, fs_list) = self.helper.find_available_qos(qos)
            if qos_id is not None:
                self.helper.add_share_to_qos(qos_id, fs_id, fs_list)
            else:
                policy_id = self.helper.create_qos_policy(qos, fs_id)
                self.helper.activate_deactivate_qos(policy_id, True)
        except exception.InvalidInput:
            with excutils.save_and_reraise_exception():
                if policy_id is not None:
                    self.helper.delete_qos_policy(policy_id)

    def _check_qos_high_priority(self, qos):
        """Check QoS priority."""
        for key, value in qos.items():
            if (key.find('MIN') == 0) or (key.find('LATENCY') == 0):
                return True

        return False

    def delete_qos(self, qos_id):
        qos_info = self.helper.get_qos_info(qos_id)
        qos_status = qos_info['RUNNINGSTATUS']
        if qos_status != constants.STATUS_QOS_INACTIVATED:
            self.helper.activate_deactivate_qos(qos_id, False)
        self.helper.delete_qos_policy(qos_id)


class SmartX(object):
    def __init__(self, helper):
        self.helper = helper

    def get_smartx_extra_specs_opts(self, opts):
        opts = self.get_capabilities_opts(opts, 'dedupe')
        opts = self.get_capabilities_opts(opts, 'compression')
        opts = self.get_smartprovisioning_opts(opts)
        opts = self.get_smartcache_opts(opts)
        opts = self.get_smartpartition_opts(opts)
        opts = self.get_sectorsize_opts(opts)
        qos = self.get_qos_opts(opts)
        return opts, qos

    def get_capabilities_opts(self, opts, key):
        if strutils.bool_from_string(opts[key]):
            opts[key] = True
        else:
            opts[key] = False

        return opts

    def get_smartprovisioning_opts(self, opts):
        thin_provision = opts.get('thin_provisioning')
        if (thin_provision is None or
                strutils.bool_from_string(thin_provision)):
            opts['LUNType'] = constants.ALLOC_TYPE_THIN_FLAG
        else:
            opts['LUNType'] = constants.ALLOC_TYPE_THICK_FLAG

        return opts

    def get_smartcache_opts(self, opts):
        if strutils.bool_from_string(opts['huawei_smartcache']):
            if not opts['cachename']:
                raise exception.InvalidInput(
                    reason=_('Cache name is None, please set '
                             'huawei_smartcache:cachename in key.'))
        else:
            opts['cachename'] = None

        return opts

    def get_smartpartition_opts(self, opts):
        if strutils.bool_from_string(opts['huawei_smartpartition']):
            if not opts['partitionname']:
                raise exception.InvalidInput(
                    reason=_('Partition name is None, please set '
                             'huawei_smartpartition:partitionname in key.'))
        else:
            opts['partitionname'] = None

        return opts

    def get_sectorsize_opts(self, opts):
        value = None
        if strutils.bool_from_string(opts.get('huawei_sectorsize')):
            value = opts.get('sectorsize')
        if not value:
            root = self.helper._read_xml()
            sectorsize = root.findtext('Filesystem/SectorSize')
            if sectorsize:
                sectorsize = sectorsize.strip()
                value = sectorsize

        if value:
            if value not in constants.VALID_SECTOR_SIZES:
                raise exception.InvalidInput(
                    reason=(_('Illegal value(%s) specified for sectorsize: '
                              'set to either 4, 8, 16, 32 or 64.') % value))
            else:
                opts['sectorsize'] = int(value)
        return opts

    def get_qos_opts(self, opts):
        qos = {}
        if not strutils.bool_from_string(opts.get('qos')):
            return

        for key, value in opts.items():
            if (key in constants.OPTS_QOS_VALUE) and value is not None:
                if (key.upper() != 'IOTYPE') and (int(value) <= 0):
                    err_msg = (_('QoS config is wrong. %(key)s'
                                 ' must be set greater than 0.')
                               % {'key': key})
                    raise exception.InvalidInput(reason=err_msg)
                elif ((key.upper() == 'IOTYPE')
                        and (value not in ['0', '1', '2'])):
                    raise exception.InvalidInput(
                        reason=(_('Illegal value specified for IOTYPE: '
                                  'set to either 0, 1, or 2.')))
                else:
                    qos[key.upper()] = value

        if len(qos) <= 1 or 'IOTYPE' not in qos:
            msg = (_('QoS config is incomplete. Please set more. '
                     'QoS policy: %(qos_policy)s.')
                   % {'qos_policy': qos})
            raise exception.InvalidInput(reason=msg)

        lowerlimit = constants.QOS_LOWER_LIMIT
        upperlimit = constants.QOS_UPPER_LIMIT
        if (set(lowerlimit).intersection(set(qos))
                and set(upperlimit).intersection(set(qos))):
            msg = (_('QoS policy conflict, both protection policy and '
                     'restriction policy are set. '
                     'QoS policy: %(qos_policy)s ')
                   % {'qos_policy': qos})
            raise exception.InvalidInput(reason=msg)

        return qos



#    Copyright (c) 2011 Justin Santa Barbara
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import log
from oslo_serialization import jsonutils
from six.moves import http_client
from six.moves.urllib import parse

LOG = log.getLogger(__name__)


class OpenStackApiException(Exception):
    def __init__(self, message=None, response=None):
        self.response = response
        if not message:
            message = 'Unspecified error'

        if response:
            _status = response.status
            _body = response.read()

            message = ('%(message)s\nStatus Code: %(_status)s\n'
                       'Body: %(_body)s') % {
                           "message": message,
                           "_status": _status,
                           "_body": _body
            }

        super(OpenStackApiException, self).__init__(message)


class OpenStackApiAuthenticationException(OpenStackApiException):
    def __init__(self, response=None, message=None):
        if not message:
            message = "Authentication error"
        super(OpenStackApiAuthenticationException, self).__init__(message,
                                                                  response)


class OpenStackApiAuthorizationException(OpenStackApiException):
    def __init__(self, response=None, message=None):
        if not message:
            message = "Authorization error"
        super(OpenStackApiAuthorizationException, self).__init__(message,
                                                                 response)


class OpenStackApiNotFoundException(OpenStackApiException):
    def __init__(self, response=None, message=None):
        if not message:
            message = "Item not found"
        super(OpenStackApiNotFoundException, self).__init__(message, response)


class TestOpenStackClient(object):
    """Simple OpenStack API Client.

    This is a really basic OpenStack API client that is under our control,
    so we can make changes / insert hooks for testing

    """

    def __init__(self, auth_user, auth_key, www_authenticate_uri):
        super(TestOpenStackClient, self).__init__()
        self.auth_result = None
        self.auth_user = auth_user
        self.auth_key = auth_key
        self.www_authenticate_uri = www_authenticate_uri
        # default project_id
        self.project_id = 'openstack'

    def request(self, url, method='GET', body=None, headers=None):
        _headers = {'Content-Type': 'application/json'}
        _headers.update(headers or {})

        parsed_url = parse.urlparse(url)
        port = parsed_url.port
        hostname = parsed_url.hostname
        scheme = parsed_url.scheme

        if scheme == 'http':
            conn = http_client.HTTPConnection(hostname,
                                              port=port)
        elif scheme == 'https':
            conn = http_client.HTTPSConnection(hostname,
                                               port=port)
        else:
            raise OpenStackApiException("Unknown scheme: %s" % url)

        relative_url = parsed_url.path
        if parsed_url.query:
            relative_url = relative_url + "?" + parsed_url.query
        LOG.info("Doing %(method)s on %(relative_url)s",
                 {"method": method, "relative_url": relative_url})
        if body:
            LOG.info("Body: %s", body)

        conn.request(method, relative_url, body, _headers)
        response = conn.getresponse()
        return response

    def _authenticate(self):
        if self.auth_result:
            return self.auth_result

        www_authenticate_uri = self.www_authenticate_uri
        headers = {'X-Auth-User': self.auth_user,
                   'X-Auth-Key': self.auth_key,
                   'X-Auth-Project-Id': self.project_id}
        response = self.request(www_authenticate_uri,
                                headers=headers)

        http_status = response.status
        LOG.debug("%(www_authenticate_uri)s => code %(http_status)s.",
                  {"www_authenticate_uri": www_authenticate_uri,
                   "http_status": http_status})

        if http_status == 401:
            raise OpenStackApiAuthenticationException(response=response)

        auth_headers = {}
        for k, v in response.getheaders():
            auth_headers[k.lower()] = v

        self.auth_result = auth_headers
        return self.auth_result

    def api_request(self, relative_uri, check_response_status=None, **kwargs):
        auth_result = self._authenticate()

        base_uri = auth_result['x-server-management-url']

        full_uri = '%s/%s' % (base_uri, relative_uri)

        headers = kwargs.setdefault('headers', {})
        headers['X-Auth-Token'] = auth_result['x-auth-token']

        response = self.request(full_uri, **kwargs)

        http_status = response.status
        LOG.debug("%(relative_uri)s => code %(http_status)s.",
                  {"relative_uri": relative_uri, "http_status": http_status})

        if check_response_status:
            if http_status not in check_response_status:
                if http_status == 404:
                    raise OpenStackApiNotFoundException(response=response)
                elif http_status == 401:
                    raise OpenStackApiAuthorizationException(response=response)
                else:
                    raise OpenStackApiException(
                        message="Unexpected status code",
                        response=response)

        return response

    def _decode_json(self, response):
        body = response.read()
        LOG.debug("Decoding JSON: %s.", (body))
        if body:
            return jsonutils.loads(body)
        else:
            return ""

    def api_options(self, relative_uri, **kwargs):
        kwargs['method'] = 'OPTIONS'
        kwargs.setdefault('check_response_status', [200])
        response = self.api_request(relative_uri, **kwargs)
        return self._decode_json(response)

    def api_get(self, relative_uri, **kwargs):
        kwargs.setdefault('check_response_status', [200])
        response = self.api_request(relative_uri, **kwargs)
        return self._decode_json(response)

    def api_post(self, relative_uri, body, **kwargs):
        kwargs['method'] = 'POST'
        if body:
            headers = kwargs.setdefault('headers', {})
            headers['Content-Type'] = 'application/json'
            kwargs['body'] = jsonutils.dumps(body)

        kwargs.setdefault('check_response_status', [200, 202])
        response = self.api_request(relative_uri, **kwargs)
        return self._decode_json(response)

    def api_put(self, relative_uri, body, **kwargs):
        kwargs['method'] = 'PUT'
        if body:
            headers = kwargs.setdefault('headers', {})
            headers['Content-Type'] = 'application/json'
            kwargs['body'] = jsonutils.dumps(body)

        kwargs.setdefault('check_response_status', [200, 202, 204])
        response = self.api_request(relative_uri, **kwargs)
        return self._decode_json(response)

    def api_delete(self, relative_uri, **kwargs):
        kwargs['method'] = 'DELETE'
        kwargs.setdefault('check_response_status', [200, 202, 204])
        return self.api_request(relative_uri, **kwargs)

    def get_shares(self, detail=True):
        rel_url = '/shares/detail' if detail else '/shares'
        return self.api_get(rel_url)['shares']



# Copyright 2014 Mirantis Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

import alembic
from alembic import config as alembic_config
import alembic.migration as alembic_migration  # pylint: disable=import-error
from oslo_config import cfg

from manila.db.sqlalchemy import api as db_api

CONF = cfg.CONF


def _alembic_config():
    path = os.path.join(os.path.dirname(__file__), os.pardir, 'alembic.ini')
    config = alembic_config.Config(path)
    return config


def version():
    """Current database version.

    :returns: Database version
    :rtype: string
    """
    engine = db_api.get_engine()
    with engine.connect() as conn:
        context = alembic_migration.MigrationContext.configure(conn)
        return context.get_current_revision()


def upgrade(revision):
    """Upgrade database.

    :param version: Desired database version
    :type version: string
    """
    return alembic.command.upgrade(_alembic_config(), revision or 'head')


def downgrade(revision):
    """Downgrade database.

    :param version: Desired database version
    :type version: string
    """
    return alembic.command.downgrade(_alembic_config(), revision or 'base')


def stamp(revision):
    """Stamp database with provided revision.

    Don't run any migrations.

    :param revision: Should match one from repository or head - to stamp
    database with most recent revision
    :type revision: string
    """
    return alembic.command.stamp(_alembic_config(), revision or 'head')


def revision(message=None, autogenerate=False):
    """Create template for migration.

    :param message: Text that will be used for migration title
    :type message: string
    :param autogenerate: If True - generates diff based on current database
    state
    :type autogenerate: bool
    """
    return alembic.command.revision(_alembic_config(), message, autogenerate)



# Copyright (c) 2011 OpenStack Foundation
# Copyright (c) 2014 NetApp, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""The share type API controller module.."""

import ast

from oslo_log import log
from oslo_utils import strutils
from oslo_utils import uuidutils
import six
from six.moves import http_client
import webob
from webob import exc

from manila.api.openstack import api_version_request as api_version
from manila.api.openstack import wsgi
from manila.api.views import types as views_types
from manila.common import constants
from manila import exception
from manila.i18n import _
from manila import rpc
from manila.share import share_types


LOG = log.getLogger(__name__)


class ShareTypesController(wsgi.Controller):
    """The share types API controller for the OpenStack API."""

    resource_name = 'share_type'
    _view_builder_class = views_types.ViewBuilder

    def __getattr__(self, key):
        if key == 'os-share-type-access':
            return self.share_type_access
        return super(ShareTypesController, self).__getattribute__(key)

    def _notify_share_type_error(self, context, method, payload):
        rpc.get_notifier('shareType').error(context, method, payload)

    def _check_body(self, body, action_name):
        if not self.is_valid_body(body, action_name):
            raise webob.exc.HTTPBadRequest()
        access = body[action_name]
        project = access.get('project')
        if not uuidutils.is_uuid_like(project):
            msg = _("Bad project format: "
                    "project is not in proper format (%s)") % project
            raise webob.exc.HTTPBadRequest(explanation=msg)

    @wsgi.Controller.authorize
    def index(self, req):
        """Returns the list of share types."""

        limited_types = self._get_share_types(req)
        req.cache_db_share_types(limited_types)
        return self._view_builder.index(req, limited_types)

    @wsgi.Controller.authorize
    def show(self, req, id):
        """Return a single share type item."""
        context = req.environ['manila.context']
        try:
            share_type = self._show_share_type_details(context, id)
        except exception.NotFound:
            msg = _("Share type not found.")
            raise exc.HTTPNotFound(explanation=msg)

        share_type['id'] = six.text_type(share_type['id'])
        req.cache_db_share_type(share_type)
        return self._view_builder.show(req, share_type)

    def _show_share_type_details(self, context, id):
        share_type = share_types.get_share_type(context, id)
        required_extra_specs = {}
        try:
            required_extra_specs = share_types.get_valid_required_extra_specs(
                share_type['extra_specs'])
        except exception.InvalidExtraSpec:
            LOG.exception('Share type %(share_type_id)s has invalid required'
                          ' extra specs.', {'share_type_id': id})

        share_type['required_extra_specs'] = required_extra_specs
        return share_type

    @wsgi.Controller.authorize
    def default(self, req):
        """Return default volume type."""
        context = req.environ['manila.context']

        try:
            share_type = share_types.get_default_share_type(context)
        except exception.NotFound:
            msg = _("Share type not found")
            raise exc.HTTPNotFound(explanation=msg)

        if not share_type:
            msg = _("Default share type not found")
            raise exc.HTTPNotFound(explanation=msg)

        share_type['id'] = six.text_type(share_type['id'])
        return self._view_builder.show(req, share_type)

    def _get_share_types(self, req):
        """Helper function that returns a list of type dicts."""
        filters = {}
        context = req.environ['manila.context']
        if context.is_admin:
            # Only admin has query access to all share types
            filters['is_public'] = self._parse_is_public(
                req.params.get('is_public'))
        else:
            filters['is_public'] = True

        extra_specs = req.params.get('extra_specs', {})
        extra_specs_disallowed = (req.api_version_request <
                                  api_version.APIVersionRequest("2.43"))

        if extra_specs and extra_specs_disallowed:
            msg = _("Filter by 'extra_specs' is not supported by this "
                    "microversion. Use 2.43 or greater microversion to "
                    "be able to use filter search by 'extra_specs.")
            raise webob.exc.HTTPBadRequest(explanation=msg)
        elif extra_specs:
            extra_specs = ast.literal_eval(extra_specs)
            filters['extra_specs'] = share_types.sanitize_extra_specs(
                extra_specs)

        limited_types = share_types.get_all_types(
            context, search_opts=filters).values()
        return list(limited_types)

    @staticmethod
    def _parse_is_public(is_public):
        """Parse is_public into something usable.

        * True: API should list public share types only
        * False: API should list private share types only
        * None: API should list both public and private share types
        """
        if is_public is None:
            # preserve default value of showing only public types
            return True
        elif six.text_type(is_public).lower() == "all":
            return None
        else:
            try:
                return strutils.bool_from_string(is_public, strict=True)
            except ValueError:
                msg = _('Invalid is_public filter [%s]') % is_public
                raise exc.HTTPBadRequest(explanation=msg)

    @wsgi.Controller.api_version("1.0", "2.23")
    @wsgi.action("create")
    def create(self, req, body):
        return self._create(req, body, set_defaults=True)

    @wsgi.Controller.api_version("2.24")  # noqa
    @wsgi.action("create")
    def create(self, req, body):  # pylint: disable=function-redefined
        return self._create(req, body, set_defaults=False)

    @wsgi.Controller.authorize('create')
    def _create(self, req, body, set_defaults=False):
        """Creates a new share type."""
        context = req.environ['manila.context']

        if (not self.is_valid_body(body, 'share_type') and
                not self.is_valid_body(body, 'volume_type')):
            raise webob.exc.HTTPBadRequest()

        elif self.is_valid_body(body, 'share_type'):
            share_type = body['share_type']
        else:
            share_type = body['volume_type']
        name = share_type.get('name')
        specs = share_type.get('extra_specs', {})
        description = share_type.get('description')
        if (description and req.api_version_request
                < api_version.APIVersionRequest("2.41")):
            msg = _("'description' key is not supported by this "
                    "microversion. Use 2.41 or greater microversion "
                    "to be able to use 'description' in share type.")
            raise webob.exc.HTTPBadRequest(explanation=msg)
        is_public = share_type.get(
            'os-share-type-access:is_public',
            share_type.get('share_type_access:is_public', True),
        )

        if (name is None or name == "" or len(name) > 255
                or (description and len(description) > 255)):
            msg = _("Type name or description is not valid.")
            raise webob.exc.HTTPBadRequest(explanation=msg)

        # Note(cknight): Set the default extra spec value for snapshot_support
        # for API versions before it was required.
        if set_defaults:
            if constants.ExtraSpecs.SNAPSHOT_SUPPORT not in specs:
                specs[constants.ExtraSpecs.SNAPSHOT_SUPPORT] = True

        try:
            required_extra_specs = (
                share_types.get_valid_required_extra_specs(specs)
            )
            share_types.create(context, name, specs, is_public,
                               description=description)
            share_type = share_types.get_share_type_by_name(context, name)
            share_type['required_extra_specs'] = required_extra_specs
            req.cache_db_share_type(share_type)
            notifier_info = dict(share_types=share_type)
            rpc.get_notifier('shareType').info(
                context, 'share_type.create', notifier_info)

        except exception.InvalidExtraSpec as e:
            raise webob.exc.HTTPBadRequest(explanation=six.text_type(e))
        except exception.ShareTypeExists as err:
            notifier_err = dict(share_types=share_type,
                                error_message=six.text_type(err))
            self._notify_share_type_error(context, 'share_type.create',
                                          notifier_err)

            raise webob.exc.HTTPConflict(explanation=six.text_type(err))
        except exception.NotFound as err:
            notifier_err = dict(share_types=share_type,
                                error_message=six.text_type(err))
            self._notify_share_type_error(context, 'share_type.create',
                                          notifier_err)
            raise webob.exc.HTTPNotFound()

        return self._view_builder.show(req, share_type)

    @wsgi.action("delete")
    @wsgi.Controller.authorize('delete')
    def _delete(self, req, id):
        """Deletes an existing share type."""
        context = req.environ['manila.context']

        try:
            share_type = share_types.get_share_type(context, id)
            share_types.destroy(context, share_type['id'])
            notifier_info = dict(share_types=share_type)
            rpc.get_notifier('shareType').info(
                context, 'share_type.delete', notifier_info)
        except exception.ShareTypeInUse as err:
            notifier_err = dict(id=id, error_message=six.text_type(err))
            self._notify_share_type_error(context, 'share_type.delete',
                                          notifier_err)
            msg = 'Target share type is still in use.'
            raise webob.exc.HTTPBadRequest(explanation=msg)
        except exception.NotFound as err:
            notifier_err = dict(id=id, error_message=six.text_type(err))
            self._notify_share_type_error(context, 'share_type.delete',
                                          notifier_err)

            raise webob.exc.HTTPNotFound()

        return webob.Response(status_int=http_client.ACCEPTED)

    @wsgi.Controller.authorize('list_project_access')
    def share_type_access(self, req, id):
        context = req.environ['manila.context']

        try:
            share_type = share_types.get_share_type(
                context, id, expected_fields=['projects'])
        except exception.ShareTypeNotFound:
            explanation = _("Share type %s not found.") % id
            raise webob.exc.HTTPNotFound(explanation=explanation)

        if share_type['is_public']:
            expl = _("Access list not available for public share types.")
            raise webob.exc.HTTPNotFound(explanation=expl)

        return self._view_builder.share_type_access(req, share_type)

    @wsgi.action('addProjectAccess')
    @wsgi.Controller.authorize('add_project_access')
    def _add_project_access(self, req, id, body):
        context = req.environ['manila.context']
        self._check_body(body, 'addProjectAccess')
        project = body['addProjectAccess']['project']

        self._verify_if_non_public_share_type(context, id)

        try:
            share_types.add_share_type_access(context, id, project)
        except exception.ShareTypeAccessExists as err:
            raise webob.exc.HTTPConflict(explanation=six.text_type(err))

        return webob.Response(status_int=http_client.ACCEPTED)

    @wsgi.action('removeProjectAccess')
    @wsgi.Controller.authorize('remove_project_access')
    def _remove_project_access(self, req, id, body):
        context = req.environ['manila.context']
        self._check_body(body, 'removeProjectAccess')
        project = body['removeProjectAccess']['project']

        self._verify_if_non_public_share_type(context, id)

        try:
            share_types.remove_share_type_access(context, id, project)
        except exception.ShareTypeAccessNotFound as err:
            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))
        return webob.Response(status_int=http_client.ACCEPTED)

    def _verify_if_non_public_share_type(self, context, share_type_id):
        try:
            share_type = share_types.get_share_type(context, share_type_id)

            if share_type['is_public']:
                msg = _("Type access modification is not applicable to "
                        "public share type.")
                raise webob.exc.HTTPConflict(explanation=msg)

        except exception.ShareTypeNotFound as err:
            raise webob.exc.HTTPNotFound(explanation=six.text_type(err))


def create_resource():
    return wsgi.Resource(ShareTypesController())



# Copyright (c) 2015 Cloudbase Solutions SRL
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from oslo_log import log


LOG = log.getLogger(__name__)


class WindowsUtils(object):
    def __init__(self, remote_execute):
        self._remote_exec = remote_execute
        self._fsutil_total_space_regex = re.compile('of bytes *: ([0-9]*)')
        self._fsutil_free_space_regex = re.compile(
            'of avail free bytes *: ([0-9]*)')

    def initialize_disk(self, server, disk_number):
        cmd = ["Initialize-Disk", "-Number", disk_number]
        self._remote_exec(server, cmd)

    def create_partition(self, server, disk_number):
        cmd = ["New-Partition", "-DiskNumber", disk_number, "-UseMaximumSize"]
        self._remote_exec(server, cmd)

    def format_partition(self, server, disk_number, partition_number):
        cmd = ("Get-Partition -DiskNumber %(disk_number)s "
               "-PartitionNumber %(partition_number)s | "
               "Format-Volume -FileSystem NTFS -Force -Confirm:$false" % {
                   'disk_number': disk_number,
                   'partition_number': partition_number,
               })
        self._remote_exec(server, cmd)

    def add_access_path(self, server, mount_path, disk_number,
                        partition_number):
        cmd = ["Add-PartitionAccessPath", "-DiskNumber", disk_number,
               "-PartitionNumber", partition_number,
               "-AccessPath", self.quote_string(mount_path)]
        self._remote_exec(server, cmd)

    def resize_partition(self, server, size_bytes, disk_number,
                         partition_number):
        cmd = ['Resize-Partition', '-DiskNumber', disk_number,
               '-PartitionNumber', partition_number,
               '-Size', size_bytes]
        self._remote_exec(server, cmd)

    def get_disk_number_by_serial_number(self, server, serial_number):
        pattern = "%s*" % serial_number[:15]
        cmd = ("Get-Disk | "
               "Where-Object {$_.SerialNumber -like '%s'} | "
               "Select-Object -ExpandProperty Number" % pattern)
        (out, err) = self._remote_exec(server, cmd)
        return int(out) if (len(out) > 0) else None

    def get_disk_number_by_mount_path(self, server, mount_path):
        cmd = ('Get-Partition | '
               'Where-Object {$_.AccessPaths -contains "%s"} | '
               'Select-Object -ExpandProperty DiskNumber' %
               (mount_path + "\\"))
        (out, err) = self._remote_exec(server, cmd)
        return int(out) if (len(out) > 0) else None

    def get_volume_path_by_mount_path(self, server, mount_path):
        cmd = ('Get-Partition | '
               'Where-Object {$_.AccessPaths -contains "%s"} | '
               'Get-Volume | '
               'Select-Object -ExpandProperty Path' %
               (mount_path + "\\"))
        (out, err) = self._remote_exec(server, cmd)
        return out.strip()

    def get_disk_space_by_path(self, server, mount_path):
        cmd = ["fsutil", "volume", "diskfree",
               self.quote_string(mount_path)]
        (out, err) = self._remote_exec(server, cmd)

        total_bytes = int(self._fsutil_total_space_regex.findall(out)[0])
        free_bytes = int(self._fsutil_free_space_regex.findall(out)[0])
        return total_bytes, free_bytes

    def get_partition_maximum_size(self, server, disk_number,
                                   partition_number):
        cmd = ('Get-PartitionSupportedSize -DiskNumber %(disk_number)s '
               '-PartitionNumber %(partition_number)s | '
               'Select-Object -ExpandProperty SizeMax' %
               dict(disk_number=disk_number,
                    partition_number=partition_number))
        (out, err) = self._remote_exec(server, cmd)

        max_bytes = int(out)
        return max_bytes

    def set_disk_online_status(self, server, disk_number, online=True):
        is_offline = int(not online)
        cmd = ["Set-Disk", "-Number", disk_number, "-IsOffline", is_offline]
        self._remote_exec(server, cmd)

    def set_disk_readonly_status(self, server, disk_number, readonly=False):
        cmd = ["Set-Disk", "-Number", disk_number,
               "-IsReadOnly", int(readonly)]
        self._remote_exec(server, cmd)

    def update_disk(self, server, disk_number):
        """Updates cached disk information."""
        cmd = ["Update-Disk", disk_number]
        self._remote_exec(server, cmd)

    def join_domain(self, server, domain, admin_username, admin_password):
        # NOTE(lpetrut): An instance reboot is needed but this will be
        # performed using Nova so that the instance state can be
        # retrieved easier.
        LOG.info("Joining server %(ip)s to Active Directory "
                 "domain %(domain)s", dict(ip=server['ip'],
                                           domain=domain))
        cmds = [
            ('$password = "%s" | '
             'ConvertTo-SecureString -asPlainText -Force' % admin_password),
            ('$credential = '
             'New-Object System.Management.Automation.PSCredential('
             '"%s", $password)' % admin_username),
            ('Add-Computer -DomainName "%s" -Credential $credential' %
             domain)]

        cmd = ";".join(cmds)
        self._remote_exec(server, cmd)

    def unjoin_domain(self, server, admin_username, admin_password,
                      reboot=False):
        cmds = [
            ('$password = "%s" | '
             'ConvertTo-SecureString -asPlainText -Force' % admin_password),
            ('$credential = '
             'New-Object System.Management.Automation.PSCredential('
             '"%s", $password)' % admin_username),
            ('Remove-Computer -UnjoinDomaincredential $credential '
             '-Passthru -Verbose -Force')]

        cmd = ";".join(cmds)
        self._remote_exec(server, cmd)

    def get_current_domain(self, server):
        cmd = "(Get-WmiObject Win32_ComputerSystem).Domain"
        (out, err) = self._remote_exec(server, cmd)
        return out.strip()

    def ensure_directory_exists(self, server, path):
        cmd = ["New-Item", "-ItemType", "Directory",
               "-Force", "-Path", self.quote_string(path)]
        self._remote_exec(server, cmd)

    def remove(self, server, path, force=True, recurse=False,
               is_junction=False):
        if self.path_exists(server, path):
            if is_junction:
                cmd = ('[System.IO.Directory]::Delete('
                       '%(path)s, %(recurse)d)'
                       % dict(path=self.quote_string(path),
                              recurse=recurse))
            else:
                cmd = ["Remove-Item", "-Confirm:$false",
                       "-Path", self.quote_string(path)]
                if force:
                    cmd += ['-Force']
                if recurse:
                    cmd += ['-Recurse']
            self._remote_exec(server, cmd)
        else:
            LOG.debug("Skipping deleting path %s as it does "
                      "not exist.", path)

    def path_exists(self, server, path):
        cmd = ["Test-Path", path]
        (out, _) = self._remote_exec(server, cmd)
        return out.strip() == "True"

    def normalize_path(self, path):
        return path.replace('/', '\\')

    def get_interface_index_by_ip(self, server, ip):
        cmd = ('Get-NetIPAddress | '
               'Where-Object {$_.IPAddress -eq "%(ip)s"} | '
               'Select-Object -ExpandProperty InterfaceIndex' %
               dict(ip=ip))

        (out, err) = self._remote_exec(server, cmd)
        if_index = int(out)
        return if_index

    def set_dns_client_search_list(self, server, search_list):
        src_list = ",".join(["'%s'" % domain for domain in search_list])

        cmd = ["Set-DnsClientGlobalSetting",
               "-SuffixSearchList", "@(%s)" % src_list]
        self._remote_exec(server, cmd)

    def set_dns_client_server_addresses(self, server, if_index, dns_servers):
        dns_sv_list = ",".join(["'%s'" % dns_sv for dns_sv in dns_servers])
        cmd = ["Set-DnsClientServerAddress",
               "-InterfaceIndex", if_index,
               "-ServerAddresses", "(%s)" % dns_sv_list]
        self._remote_exec(server, cmd)

    def set_win_reg_value(self, server, path, key, value):
        cmd = ['Set-ItemProperty', '-Path', self.quote_string(path),
               '-Name', key, '-Value', value]
        self._remote_exec(server, cmd)

    def get_win_reg_value(self, server, path, name=None):
        cmd = "Get-ItemProperty -Path %s" % self.quote_string(path)
        if name:
            cmd += " | Select-Object -ExpandProperty %s" % name
        return self._remote_exec(server, cmd, retry=False)[0]

    def quote_string(self, string):
        return '"%s"' % string



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import copy
import datetime

import ddt
import mock
from oslo_config import cfg
import webob

from manila.api.v2 import share_group_types as types
from manila import exception
from manila import policy
from manila.share_group import share_group_types
from manila import test
from manila.tests.api import fakes

CONF = cfg.CONF

PROJ1_UUID = '11111111-1111-1111-1111-111111111111'
PROJ2_UUID = '22222222-2222-2222-2222-222222222222'
PROJ3_UUID = '33333333-3333-3333-3333-333333333333'

SHARE_TYPE_ID = '4b1e460f-8bc5-4a97-989b-739a2eceaec6'
GROUP_TYPE_1 = {
    'id': 'c8d7bf70-0db9-4b3e-8498-055dd0306461',
    'name': u'group type 1',
    'deleted': False,
    'created_at': datetime.datetime(2012, 1, 1, 1, 1, 1, 1),
    'updated_at': None,
    'deleted_at': None,
    'is_public': True,
    'group_specs': {},
    'share_types': [],
}

GROUP_TYPE_2 = {
    'id': 'f93f7a1f-62d7-4e7e-b9e6-72eec95a47f5',
    'name': u'group type 2',
    'deleted': False,
    'created_at': datetime.datetime(2012, 1, 1, 1, 1, 1, 1),
    'updated_at': None,
    'deleted_at': None,
    'is_public': False,
    'group_specs': {'consistent_snapshots': 'true'},
    'share_types': [{'share_type_id': SHARE_TYPE_ID}],
}

GROUP_TYPE_3 = {
    'id': '61fdcbed-db27-4cc0-8938-8b4f74c2ae59',
    'name': u'group type 3',
    'deleted': False,
    'created_at': datetime.datetime(2012, 1, 1, 1, 1, 1, 1),
    'updated_at': None,
    'deleted_at': None,
    'is_public': True,
    'group_specs': {},
    'share_types': [],
}


def fake_request(url, admin=False, experimental=True, version='2.31',
                 **kwargs):

    return fakes.HTTPRequest.blank(
        url,
        use_admin_context=admin,
        experimental=experimental,
        version=version,
        **kwargs
    )


@ddt.ddt
class ShareGroupTypesAPITest(test.TestCase):

    def setUp(self):
        super(ShareGroupTypesAPITest, self).setUp()
        self.flags(host='fake')
        self.controller = types.ShareGroupTypesController()
        self.resource_name = self.controller.resource_name
        self.mock_object(policy, 'check_policy', mock.Mock(return_value=True))

    def test_share_group_types_index(self):
        fake_types = {GROUP_TYPE_1['name']: GROUP_TYPE_1}
        mock_get_all = self.mock_object(
            share_group_types, 'get_all', mock.Mock(return_value=fake_types))
        req = fake_request('/v2/fake/share-group-types', admin=False)
        expected_list = [{
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': True,
            'group_specs': {},
            'share_types': [],
        }]

        res_dict = self.controller.index(req)

        mock_get_all.assert_called_once_with(
            mock.ANY, search_opts={"is_public": True})
        self.assertEqual(1, len(res_dict['share_group_types']))
        self.assertEqual(expected_list, res_dict['share_group_types'])

    def test_share_group_types_index_as_admin(self):
        fake_types = {
            GROUP_TYPE_1['name']: GROUP_TYPE_1,
            GROUP_TYPE_2['name']: GROUP_TYPE_2,
        }
        mock_get_all = self.mock_object(
            share_group_types, 'get_all',
            mock.Mock(return_value=fake_types))
        req = fake_request(
            '/v2/fake/share-group-types?is_public=all', admin=True)
        expected_type_1 = {
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': True,
            'group_specs': {},
            'share_types': [],
        }
        expected_type_2 = {
            'id': GROUP_TYPE_2['id'],
            'name': GROUP_TYPE_2['name'],
            'is_public': False,
            'group_specs': {'consistent_snapshots': 'true'},
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller.index(req)

        mock_get_all.assert_called_once_with(
            mock.ANY, search_opts={'is_public': None})
        self.assertEqual(2, len(res_dict['share_group_types']))
        self.assertIn(expected_type_1, res_dict['share_group_types'])
        self.assertIn(expected_type_2, res_dict['share_group_types'])

    def test_share_group_types_index_as_admin_default_public_only(self):
        fake_types = {}
        mock_get_all = self.mock_object(
            share_group_types, 'get_all',
            mock.Mock(return_value=fake_types))
        req = fake_request('/v2/fake/share-group-types', admin=True)

        self.controller.index(req)

        mock_get_all.assert_called_once_with(
            mock.ANY, search_opts={'is_public': True})

    def test_share_group_types_index_not_experimental(self):
        self.mock_object(
            share_group_types, 'get_all', mock.Mock(return_value={}))
        req = fake_request('/v2/fake/share-group-types', experimental=False)

        self.assertRaises(
            exception.VersionNotFoundForAPIMethod, self.controller.index, req)
        self.assertFalse(share_group_types.get_all.called)

    def test_share_group_types_index_older_api_version(self):
        self.mock_object(
            share_group_types, 'get_all', mock.Mock(return_value={}))
        req = fake_request('/v2/fake/share-group-types', version='2.1')

        self.assertRaises(
            exception.VersionNotFoundForAPIMethod, self.controller.index, req)

    @ddt.data(True, False)
    def test_share_group_types_index_no_data(self, admin):
        self.mock_object(
            share_group_types, 'get_all', mock.Mock(return_value={}))
        req = fake_request('/v2/fake/share-group-types', admin=admin)

        res_dict = self.controller.index(req)

        self.assertEqual(0, len(res_dict['share_group_types']))

    def test_share_group_types_show(self):
        mock_get = self.mock_object(
            share_group_types, 'get',
            mock.Mock(return_value=GROUP_TYPE_1))
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_1['id'])
        expected_type = {
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': True,
            'group_specs': {},
            'share_types': [],
        }

        res_dict = self.controller.show(req, GROUP_TYPE_1['id'])

        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_1['id'])
        self.assertEqual(expected_type, res_dict['share_group_type'])

    def test_share_group_types_show_with_share_types(self):
        mock_get = self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_2))
        req = fake_request('/v2/fake/group-types/%s' % GROUP_TYPE_2['id'])
        expected_type = {
            'id': GROUP_TYPE_2['id'],
            'name': GROUP_TYPE_2['name'],
            'is_public': False,
            'group_specs': {'consistent_snapshots': 'true'},
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller.show(req, GROUP_TYPE_2['id'])

        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_2['id'])
        self.assertEqual(expected_type, res_dict['share_group_type'])

    def test_share_group_types_show_not_found(self):
        mock_get = self.mock_object(
            share_group_types, 'get',
            mock.Mock(side_effect=exception.ShareGroupTypeNotFound(
                type_id=GROUP_TYPE_2['id'])))
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'])

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.show, req, GROUP_TYPE_2['id'])

        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_2['id'])

    def test_share_group_types_default(self):
        mock_get = self.mock_object(
            share_group_types, 'get_default',
            mock.Mock(return_value=GROUP_TYPE_2))
        req = fake_request('/v2/fake/share-group-types/default')
        expected_type = {
            'id': GROUP_TYPE_2['id'],
            'name': GROUP_TYPE_2['name'],
            'is_public': False,
            'group_specs': {'consistent_snapshots': 'true'},
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller.default(req)

        mock_get.assert_called_once_with(mock.ANY)
        self.assertEqual(expected_type, res_dict['share_group_type'])

    def test_share_group_types_default_not_found(self):
        mock_get = self.mock_object(
            share_group_types, 'get_default', mock.Mock(return_value=None))
        req = fake_request('/v2/fake/share-group-types/default')

        self.assertRaises(webob.exc.HTTPNotFound, self.controller.default, req)

        mock_get.assert_called_once_with(mock.ANY)

    def test_share_group_types_delete(self):
        mock_get = self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_1))
        mock_destroy = self.mock_object(share_group_types, 'destroy')
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_1['id'])

        self.controller._delete(req, GROUP_TYPE_1['id'])

        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_1['id'])
        mock_destroy.assert_called_once_with(mock.ANY, GROUP_TYPE_1['id'])

    def test_share_group_types_delete_not_found(self):
        mock_get = self.mock_object(
            share_group_types, 'get',
            mock.Mock(side_effect=exception.ShareGroupTypeNotFound(
                type_id=GROUP_TYPE_2['id'])))
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'])

        self.assertRaises(webob.exc.HTTPNotFound, self.controller._delete,
                          req, GROUP_TYPE_2['id'])

        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_2['id'])

    def test_create_minimal(self):
        fake_type = copy.deepcopy(GROUP_TYPE_1)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        mock_create = self.mock_object(share_group_types, 'create')
        mock_get = self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
        }}
        expected_type = {
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': True,
            'group_specs': {},
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller._create(req, fake_body)

        mock_create.assert_called_once_with(
            mock.ANY, GROUP_TYPE_1['name'],
            [SHARE_TYPE_ID], {}, True)
        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_1['name'])
        self.assertEqual(expected_type, res_dict['share_group_type'])

    @ddt.data(
        None, {'my_fake_group_spec': 'false'},
    )
    def test_create_with_group_specs(self, specs):
        fake_type = copy.deepcopy(GROUP_TYPE_1)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        fake_type['group_specs'] = specs
        mock_create = self.mock_object(share_group_types, 'create')
        mock_get = self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
            'group_specs': specs,
        }}
        expected_type = {
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': True,
            'group_specs': specs,
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller._create(req, fake_body)

        mock_create.assert_called_once_with(
            mock.ANY, GROUP_TYPE_1['name'], [SHARE_TYPE_ID], specs,
            True)
        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_1['name'])
        self.assertEqual(expected_type, res_dict['share_group_type'])

    @ddt.data(
        'str', ['l', 'i', 's', 't'], set([1]), ('t', 'u', 'p', 'l', 'e'), 1,
        {"foo": 1}, {1: "foo"}, {"foo": "bar", "quuz": []}
    )
    def test_create_with_wrong_group_specs(self, specs):
        fake_type = copy.deepcopy(GROUP_TYPE_1)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        fake_type['group_specs'] = specs
        mock_create = self.mock_object(share_group_types, 'create')
        mock_get = self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
            'group_specs': specs,
        }}

        self.assertRaises(
            webob.exc.HTTPBadRequest, self.controller._create, req, fake_body)

        self.assertEqual(0, mock_create.call_count)
        self.assertEqual(0, mock_get.call_count)

    def test_create_private_share_group_type(self):
        fake_type = copy.deepcopy(GROUP_TYPE_1)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        fake_type['is_public'] = False
        mock_create = self.mock_object(share_group_types, 'create')
        mock_get = self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
            'is_public': False
        }}
        expected_type = {
            'id': GROUP_TYPE_1['id'],
            'name': GROUP_TYPE_1['name'],
            'is_public': False,
            'group_specs': {},
            'share_types': [SHARE_TYPE_ID],
        }

        res_dict = self.controller._create(req, fake_body)

        mock_create.assert_called_once_with(
            mock.ANY, GROUP_TYPE_1['name'], [SHARE_TYPE_ID], {}, False)
        mock_get.assert_called_once_with(mock.ANY, GROUP_TYPE_1['name'])
        self.assertEqual(expected_type, res_dict['share_group_type'])

    def test_create_invalid_request_duplicate_name(self):
        mock_create = self.mock_object(
            share_group_types, 'create',
            mock.Mock(side_effect=exception.ShareGroupTypeExists(
                type_id=GROUP_TYPE_1['name'])))
        req = fake_request('/v2/fake/sahre-group-types')
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
        }}

        self.assertRaises(
            webob.exc.HTTPConflict, self.controller._create, req, fake_body)

        mock_create.assert_called_once_with(
            mock.ANY, GROUP_TYPE_1['name'], [SHARE_TYPE_ID], {}, True)

    def test_create_invalid_request_missing_name(self):
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {'share_types': [SHARE_TYPE_ID]}}

        self.assertRaises(
            webob.exc.HTTPBadRequest, self.controller._create, req, fake_body)

    def test_create_invalid_request_missing_share_types(self):
        req = fake_request('/v2/fake/share-group-types')
        fake_body = {'share_group_type': {'name': GROUP_TYPE_1['name']}}

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller._create, req, fake_body)

    def test_create_provided_share_type_does_not_exist(self):
        req = fake_request('/v2/fake/share-group-types', admin=True)
        fake_body = {
            'share_group_type': {
                'name': GROUP_TYPE_1['name'],
                'share_types': SHARE_TYPE_ID + '_does_not_exist',
            }
        }

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller._create, req, fake_body)

    @ddt.data(('2.45', True), ('2.45', False),
              ('2.46', True), ('2.46', False))
    @ddt.unpack
    def test_share_group_types_create_with_is_default_key(self,
                                                          version,
                                                          admin):
        # is_default is false
        fake_type = copy.deepcopy(GROUP_TYPE_1)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        self.mock_object(share_group_types, 'create')
        self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types',
                           version=version,
                           admin=admin)
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_1['name'],
            'share_types': [SHARE_TYPE_ID],
        }}
        res_dict = self.controller._create(req, fake_body)
        if self.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', res_dict['share_group_type'])
            self.assertIs(False, res_dict['share_group_type']['is_default'])
        else:
            self.assertNotIn('is_default', res_dict['share_group_type'])

        # is_default is true
        default_type_name = 'group type 3'
        CONF.set_default('default_share_group_type', default_type_name)

        fake_type = copy.deepcopy(GROUP_TYPE_3)
        fake_type['share_types'] = [{'share_type_id': SHARE_TYPE_ID}]
        self.mock_object(share_group_types, 'create')
        self.mock_object(
            share_group_types, 'get_by_name',
            mock.Mock(return_value=fake_type))
        req = fake_request('/v2/fake/share-group-types',
                           version=version,
                           admin=admin)
        fake_body = {'share_group_type': {
            'name': GROUP_TYPE_3['name'],
            'share_types': [SHARE_TYPE_ID],
        }}
        res_dict = self.controller._create(req, fake_body)
        if self.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', res_dict['share_group_type'])
            self.assertIs(True, res_dict['share_group_type']['is_default'])
        else:
            self.assertNotIn('is_default', res_dict['share_group_type'])

    @ddt.data(('2.45', True), ('2.45', False),
              ('2.46', True), ('2.46', False))
    @ddt.unpack
    def test_share_group_types_list_with_is_default_key(self, version, admin):
        fake_types = {
            GROUP_TYPE_1['name']: GROUP_TYPE_1,
            GROUP_TYPE_2['name']: GROUP_TYPE_2,
        }
        self.mock_object(
            share_group_types, 'get_all',
            mock.Mock(return_value=fake_types))
        req = fake_request(
            '/v2/fake/share-group-types?is_public=all',
            version=version,
            admin=admin)
        res_dict = self.controller.index(req)
        for res in res_dict['share_group_types']:
            if self.is_microversion_ge(version, '2.46'):
                self.assertIn('is_default', res)
                self.assertIs(False, res['is_default'])
            else:
                self.assertNotIn('is_default', res)
        self.assertEqual(2, len(res_dict['share_group_types']))

    @ddt.data(('2.45', True), ('2.45', False),
              ('2.46', True), ('2.46', False))
    @ddt.unpack
    def test_shares_group_types_show_with_is_default_key(self, version, admin):
        self.mock_object(
            share_group_types, 'get',
            mock.Mock(return_value=GROUP_TYPE_2))
        req = fake_request('/v2/fake/group-types/%s' % GROUP_TYPE_2['id'],
                           version=version,
                           admin=admin)
        res_dict = self.controller.show(req, GROUP_TYPE_2['id'])
        if self.is_microversion_ge(version, '2.46'):
            self.assertIn('is_default', res_dict['share_group_type'])
            self.assertIs(False, res_dict['share_group_type']['is_default'])
        else:
            self.assertNotIn('is_default', res_dict['share_group_type'])


class ShareGroupTypeAccessTest(test.TestCase):

    def setUp(self):
        super(ShareGroupTypeAccessTest, self).setUp()
        self.controller = types.ShareGroupTypesController()

    def test_list_type_access_public(self):
        self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_1))
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_1['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.share_group_type_access, req, GROUP_TYPE_1['id'])

    def test_list_type_access_private(self):
        fake_type = copy.deepcopy(GROUP_TYPE_2)
        fake_type['projects'] = [PROJ2_UUID, PROJ3_UUID]
        mock_get = self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=fake_type))
        expected = {'share_group_type_access': [
            {'share_group_type_id': fake_type['id'], 'project_id': PROJ2_UUID},
            {'share_group_type_id': fake_type['id'], 'project_id': PROJ3_UUID},
        ]}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % fake_type['id'], admin=True)

        actual = self.controller.share_group_type_access(req, fake_type['id'])

        mock_get.assert_called_once_with(
            mock.ANY, fake_type['id'], expected_fields=['projects'])
        self.assertEqual(expected, actual)

    def test_list_type_access_type_not_found(self):
        self.mock_object(
            share_group_types, 'get',
            mock.Mock(side_effect=exception.ShareGroupTypeNotFound(
                type_id=GROUP_TYPE_2['id'])))
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller.share_group_type_access, req, GROUP_TYPE_2['id'])

    def test_add_project_access(self):
        self.mock_object(share_group_types, 'get',
                         mock.Mock(return_value=GROUP_TYPE_2))
        mock_add_access = self.mock_object(
            share_group_types, 'add_share_group_type_access')
        body = {'addProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        response = self.controller._add_project_access(
            req, GROUP_TYPE_2['id'], body)

        mock_add_access.assert_called_once_with(
            mock.ANY, GROUP_TYPE_2['id'], PROJ1_UUID)
        self.assertEqual(202, response.status_code)

    def test_add_project_access_non_existent_type(self):
        self.mock_object(
            share_group_types, 'get',
            mock.Mock(side_effect=exception.ShareGroupTypeNotFound(
                type_id=GROUP_TYPE_2['id'])))
        body = {'addProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller._add_project_access, req, GROUP_TYPE_2['id'], body)

    def test_add_project_access_missing_project_in_body(self):
        body = {'addProjectAccess': {}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller._add_project_access, req, GROUP_TYPE_2['id'], body)

    def test_add_project_access_missing_add_project_access_in_body(self):
        body = {}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPBadRequest,
            self.controller._add_project_access, req, GROUP_TYPE_2['id'], body)

    def test_add_project_access_with_already_added_access(self):
        self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_2))
        mock_add_access = self.mock_object(
            share_group_types, 'add_share_group_type_access',
            mock.Mock(side_effect=exception.ShareGroupTypeAccessExists(
                type_id=GROUP_TYPE_2['id'], project_id=PROJ1_UUID))
        )
        body = {'addProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPConflict,
            self.controller._add_project_access, req, GROUP_TYPE_2['id'], body)

        mock_add_access.assert_called_once_with(
            mock.ANY, GROUP_TYPE_2['id'], PROJ1_UUID)

    def test_add_project_access_to_public_share_type(self):
        self.mock_object(
            share_group_types, 'get',  mock.Mock(return_value=GROUP_TYPE_1))
        body = {'addProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_1['id'], admin=True)

        self.assertRaises(
            webob.exc.HTTPConflict,
            self.controller._add_project_access, req, GROUP_TYPE_1['id'], body)

    def test_remove_project_access(self):
        self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_2))
        mock_remove_access = self.mock_object(
            share_group_types, 'remove_share_group_type_access')
        body = {'removeProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        response = self.controller._remove_project_access(
            req, GROUP_TYPE_2['id'], body)

        mock_remove_access.assert_called_once_with(
            mock.ANY, GROUP_TYPE_2['id'], PROJ1_UUID)
        self.assertEqual(202, response.status_code)

    def test_remove_project_access_nonexistent_rule(self):
        self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_2))
        mock_remove_access = self.mock_object(
            share_group_types, 'remove_share_group_type_access', mock.Mock(
                side_effect=exception.ShareGroupTypeAccessNotFound(
                    type_id=GROUP_TYPE_2['id'], project_id=PROJ1_UUID)))
        body = {'removeProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request('/v2/fake/group-types/%s' % GROUP_TYPE_2['id'],
                           admin=True)

        self.assertRaises(
            webob.exc.HTTPNotFound,
            self.controller._remove_project_access,
            req, GROUP_TYPE_2['id'], body)

        mock_remove_access.assert_called_once_with(
            mock.ANY, GROUP_TYPE_2['id'], PROJ1_UUID)

    def test_remove_project_access_from_public_share_type(self):
        self.mock_object(
            share_group_types, 'get', mock.Mock(return_value=GROUP_TYPE_1))
        body = {'removeProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_1['id'], admin=True)

        self.assertRaises(webob.exc.HTTPConflict,
                          self.controller._remove_project_access, req,
                          GROUP_TYPE_1['id'], body)

    def test_remove_project_access_non_existent_type(self):
        self.mock_object(
            share_group_types, 'get',
            mock.Mock(side_effect=exception.ShareGroupTypeNotFound(
                type_id=GROUP_TYPE_2['id'])))
        body = {'removeProjectAccess': {'project': PROJ1_UUID}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(webob.exc.HTTPNotFound,
                          self.controller._remove_project_access, req,
                          GROUP_TYPE_2['id'], body)

    def test_remove_project_access_missing_project_in_body(self):
        body = {'removeProjectAccess': {}}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._remove_project_access, req,
                          GROUP_TYPE_2['id'], body)

    def test_remove_project_access_missing_remove_project_access_in_body(self):
        body = {}
        req = fake_request(
            '/v2/fake/share-group-types/%s' % GROUP_TYPE_2['id'], admin=True)

        self.assertRaises(webob.exc.HTTPBadRequest,
                          self.controller._remove_project_access, req,
                          GROUP_TYPE_2['id'], body)
